/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.army.entities;

import java.awt.geom.Area;

import fb.battlefield.Sprite;
import fb.main.FB;
import fb.network.Client;
import fb.network.MessagesConstants;
import fb.util.StringTools;

/**
 * delegate class for flying methods used by entities 
 * that are both shooting and flying entities (double heritage is not possible in java)
 * @author farfpeuf
 */
public class FlyingEntityDelegate implements FlyingEntity,Cloneable{
/**
 * is entity currently flying
 */
	private boolean isFlying=true;
	/**
	 * sprites used when entity is flying
	 */
	private Sprite flyingSprite,shadowSprite;
	/**
	 * entity for which this is delegate
	 */
	private Entity entity;
	/**
	 * reset unit after end of battle
	 * 
	 * @param winner
	 *            did the unit army won the battle
	 */
	public void endBattle(boolean winner) {
		isFlying=true;
	}
	/**
	 * constructor
	 * @param entity entity for which this is delegate
	 */
	public FlyingEntityDelegate(Entity entity){
		this.entity=entity;
	}
	/**
	 * @return sprite used when flying
	 */
	public Sprite getFlyingSprite() {
		return flyingSprite;
	}
	/**
	 * create entity sprites
	 * 
	 */
	public void createsprite() {
		flyingSprite= Sprite.getInstance(entity,"f");
		shadowSprite= Sprite.getInstance(entity,"s");

	}

	/**
	 * deletes sprite to free memory
	 * 
	 */
    public void freeSprite() {
        flyingSprite= null;
        shadowSprite= null;

    }	
	/**
	 * @return shadow sprite used when flying
	 */
	public Sprite getShadowSprite() {
		return shadowSprite;
	}

	/**
	 * returns delegate as a string to save entity to a file
	 * 
	 * @return delegate as a string
	 */
	public String savestring() {
		return "\t" + (isFlying? 'y':'n');

	}

	/**
	 * load delegate from a StringBuffer
	 * 
	 * @param buffy
	 *            the StringBuffer
	 */
	public void loadString(StringBuffer buffy) {
		isFlying=(StringTools.readBufferChar(buffy)=='y')? true:false;
	}

	/**
	 * is this entity currently flying
	 * 
	 * @return true if this entity currently flying
	 */
	public boolean isFlying() {
		return isFlying;
	}
	/**
	 * can this entity do a take off manoeuvre
	 * 
	 * @return true if it can
	 */
	public boolean canTakeOff() {
		if(!entity.isMoveAble()|| isFlying() || entity.getRemainingMove()<=0){
			return false;
		}
		Area obstacles;
		obstacles = entity.getMovementObstacles(false);
		obstacles.intersect(entity.area);
		return obstacles.isEmpty();
	}	
	/**
	 * can this entity do a land manoeuvre
	 * 
	 * @return true if it can
	 */
	public boolean canLand() {
		if(!entity.isMoveAble()|| !isFlying() || entity.getRemainingMove()<=0){
			return false;
		}
		Area obstacles;
		obstacles = entity.getMovementObstacles(true);
		obstacles.intersect(entity.area);
		return obstacles.isEmpty();
	}	
	/**
	 * do take off manoeuvre
	 * 
	 */
	public void doTakeOff() {
		entity.consumeMovement(1);
		isFlying=true;
		entity.firePropertyChange(Entity.FLYING,false,true);
	}	
	/**
	 * do land manoeuvre
	 * 
	 */
	public void doLand() {
		entity.consumeMovement(1);
		isFlying=false;
		entity.firePropertyChange(Entity.FLYING,true,false);
	}		
	/**
	 * take off manoeuvre
	 * 
	 */
	public void takeOff() {
		if (FB.isNet()) {
			Client.getInstance().sendAndWait(MessagesConstants.TAKEOFF + entity.getId());
		} else {
			doTakeOff();
		}
	}	
	/**
	 * land manoeuvre
	 * 
	 */
	public void land() {
		if (FB.isNet()) {
			Client.getInstance().sendAndWait(MessagesConstants.LAND + entity.getId());
		} else {
			doLand();
		}
	}	
	/**
	 * clone this delegate
	 */
	public Object clone(Entity entity) {
		try {
			Object clone=super.clone();
			((FlyingEntityDelegate)clone).entity=entity;
			return clone;
		} catch (CloneNotSupportedException cnse) {
			cnse.printStackTrace();
			return null;
		}
	}
	/**
	 * take off or land to be on the same level as another entity
	 * @param target target entity
	 */
	public void takeOffLandTo(Entity target) {
		if(target!=null && !entity.sameAltitude(target)){
			if(entity.isFlying() && ((FlyingEntity)entity).canLand()){
					((FlyingEntity)entity).land();
			}else if(entity.stepsTo(target)<=entity.getRemainingMove() && ((FlyingEntity)entity).canTakeOff()){
					((FlyingEntity)entity).takeOff();
			}
			
		}
	}	
	
}
