/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.army.entities;

import fb.army.hierarchy.Command;
import fb.army.hierarchy.MercenaryCommand;
import fb.battlefieldelements.SignWithValue;
import fb.main.FB;
import fb.util.D100;
import fb.util.Resources;
import fb.util.StringTools;

/**
 * a command leader
 * 
 * @author farfpeuf
 * 
 */
public class Leader extends MessageReceiver {
	/**
	 * leadership
	 */
	private int leadership = 3;

	/**
	 * messenger attached to leader
	 */
	private MessageReceiver messenger;

	/**
	 * additionnal to display leadership value
	 */
	private SignWithValue leadershipSign;

	/**
	 * create inidividual from line read in catalog
	 * 
	 * @param ligne
	 *            catalog line
	 * @param linenumber
	 *            line number
	 */
	public Leader(String ligne, int linenumber) {
		super(ligne, linenumber);
	}

	/**
	 * initialize entity at the beginning of a battle
	 * 
	 */
	public void initBattle() {
		super.initBattle();
		leadershipSign = new SignWithValue(this, leadership);
		leadershipSign.setImage(Resources.getSign("leadershipcounter"));
		signs.add(1, leadershipSign);
	}

	/**
	 * reset entity after end of battle
	 * 
	 * @param winner
	 *            did the entity army won the battle
	 */
	public void endBattle(boolean winner) {
		super.endBattle(winner);
		leadershipSign = null;
		messenger = null;
	}

	/**
	 * reset remaining movement points
	 * 
	 */
	public void resetRemainingMove() {
		super.resetRemainingMove();
		command.resetChangedOrders();
	}


	/**
	 * calculate entity cost
	 */
	public int getCost() {
		int leadershipcost;
		if (leadership == 1) {
			leadershipcost = 10;
		} else if (leadership == 2) {
			leadershipcost = 20;
		} else if (leadership == 3) {
			leadershipcost = 30;
		} else if (leadership == 4) {
			leadershipcost = 40;
		} else {
			leadershipcost = 50;
		}
		return price + leadershipcost;
	}

	/**
	 * returns individual as a string to save it to a file
	 * 
	 * @return individual as a string
	 */
	public String saveString() {
		return super.saveString() + '\t' + leadership;
	}

	/**
	 * load individual from a StringBuffer
	 * 
	 * @param buffy
	 *            the StringBuffer
	 */
	public void loadString(StringBuffer buffy) {
		super.loadString(buffy);
		leadership = StringTools.readBufferInt(buffy);
	}

	/**
	 * calculate orders range
	 * 
	 * @return maximum orders range
	 */
	public int orderRange() {
		int range = 10;
		Entity entity;
		if (nearBanner()) {
			range += (16);
		}
		for (int i = 0; i < getArmy().getActiveEntities().size(); i++) {
			entity = (Entity) (getArmy().getActiveEntities().get(i));
			if (entity instanceof Herald && near(entity)) {
				range += (8);
			}
		}
		return range;
	}

	/**
	 * is order distance ok for all command units
	 * 
	 * @return true if order distance is ok for all command units
	 */
	public boolean orderDistanceOkForAll() {
		int range = orderRange();
		Unit unit;
		for (int i = 0; i < command.getUnits().size(); i++) {
			unit = (Unit) (command.getUnits().get(i));
			if (stepsTo(unit) > range && unit.getState() != 'r'
					&& unit.getState() != 'a') {
				return false;
			}
		}
		return true;
	}

	/**
	 * receive new orders by magic
	 * 
	 * @param no
	 *            new orders
	 * @return true if orders are accepted
	 */
	public boolean magicNewOrder(char no) {
		String message = Resources.getString("leaderneworder");

		message = StringTools.replaceAll(message, "<entity full name>",
				fullName);
		FB.getInstance().message(message, getColor());
		int score = D100.roll() + leadership * 15;

		if (race != getArmy().getWarchief().getRace()) {
			score -= 15;
		}
		if (suffersBadLight()) {
			score -= 15;
		}
		if (command.hasDisorganizedUnits()) {
			score -= 15;
		}
		if (command instanceof MercenaryCommand) {
				score -= 15;
		}
		score -= (command.nbShakenUnits() * 15);
		score -= (command.nbBloodlustUnits() * 15);
		if ((no == 'a') && (command.isFanatic())) {
			score += 15;
		}
		if (score >= 80) {
			command.changeOrder(no);
			return true;
		} else if (score > 65) {
			command.changeOrder(no);
			command.setDisorganized();
			return true;
		} else if (score >= 50) {
			command.setDisorganized();
		} else if (score >= 0) {
			if (no == 'h') {
				command.changeOrder('a');
			} else {
				command.changeOrder('h');
			}
		} else {
			command.setShaken();
		}
		return false;
	}

	/**
	 * accept new orders
	 * 
	 */
	public boolean accceptNewOrders() {
		String message = Resources.getString("leaderneworder");
		message = StringTools.replaceAll(message, "<entity full name>",
				fullName);
		FB.getInstance().message(message, getColor());

		command.changeOrder(newOrder);
		return true;

	}

	/**
	 * is leader near a banner
	 * 
	 * @return true if leader is near a banner
	 */
	public boolean nearBanner() {
		Entity entity;
		for (int i = 0; i < getArmy().getActiveEntities().size(); i++) {
			entity = (Entity) (getArmy().getActiveEntities().get(i));
			if (entity instanceof Banner && near(entity)) {
				return true;
			}
		}
		return false;
	}

	/**
	 * @return rsource key
	 */
	protected String getResourceKey() {
		if (getArmy() != null && getArmy().getWarchief() == this) {
			return "warchief";
		} else {
			return "leader";
		}
	}



	/**
	 * @return Returns the messenger.
	 */
	public MessageReceiver getMessenger() {
		return messenger;
	}

	/**
	 * @param messenger The messenger to set.
	 */
	public void setMessenger(MessageReceiver messenger) {
		this.messenger = messenger;
	}


	/**
	 * @return Returns the leadership.
	 */
	public int getLeadership() {
		return leadership;
	}

	/**
	 * @param leadership The leadership to set.
	 */
	public void setLeadership(int leadership) {
		this.leadership = leadership;
	}
	/**
	 * @param group
	 *            The group to set.
	 */
	public void setCommand(Command group) {
		super.setCommand(group);
		if (group != null) {
			group.setLeader(this);
		}
	}

}