/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.army.entities;

import java.awt.Color;
import java.awt.geom.Area;
import java.util.ArrayList;
import java.util.List;

import fb.battlefield.BattleFieldModel;
import fb.battlefield.Sprite;
import fb.battlefield.PaintedWithSprite;
import fb.battlefieldelements.Figure;
import fb.battlefieldelements.Projectile;
import fb.battlefieldelements.Warrior;

/**
 * war machines attached to crew
 * 
 * @author farfpeuf
 * 
 */
public class WarMachine implements Cloneable, PaintedWithSprite {

	/**
	 * additionnal characteristics
	 */
	private int strength, worth, movement, normalCrewNumber, max, min, price,
			shortrange, midRange, longrange, shootingRows, number,
			initialNumber;

	/**
	 * machine sprite number
	 */
	private int spriteNumber;

	/**
	 * machine size
	 */
	private float width, length;

	/**
	 * machine models
	 */
	private List models;

	/**
	 * machine base area
	 */
	private Area area;

	/**
	 * machine sprite
	 */
	private Sprite sprite;

	/**
	 * crew to which these machines are attached
	 */
	private Crew crew;

	/**
	 * type of projetcile shot
	 */
	private char projectileType;

	/**
	 * constructor
	 * 
	 * @param type
	 *            machine type
	 * @param crew
	 *            crew to which these machines are attached
	 */
	public WarMachine(char type, Crew crew) {
		this.crew = crew;

		if (type == 'c') {
			shortrange = 6;
			longrange = 40;
			shootingRows = 1;
			strength = 5;
			worth = 5;
			movement = 3;
			max = 3;
			min = 1;
			price = 100;
			width = 2;
			length = 2;
			spriteNumber = 200;
			normalCrewNumber = 2;
			projectileType = 'b';
		} else if (type == 'g') {
			shortrange = 6;
			longrange = 40;
			shootingRows = 2;
			strength = 2;
			worth = 3;
			movement = 4;
			max = 5;
			min = 2;
			price = 30;
			width = 1;
			length = 1.5f;
			spriteNumber = 201;
			normalCrewNumber = 2;
			projectileType = 's';
		} else if (type == 't') {
			shortrange = 0;
			longrange = 60;
			shootingRows = 2;
			strength = 6;
			worth = 7;
			movement = 2;
			max = 3;
			min = 1;
			price = 270;
			width = 3;
			length = 4f;
			spriteNumber = 200;
			normalCrewNumber = 5;
			projectileType = 'b';
		}

		midRange = shortrange + longrange / 2;
	}

	/**
	 * orientate all the machines
	 * 
	 */
	public void orientate(double orientation, int x, int y) {
		Figure model;
		for (int i = 0; i < number; i++) {
			model = getModel(i);
			model.orientate(orientation, x, y);
			model.setAngle(orientation);
		}
	}

	/**
	 * calculate base area
	 * 
	 */
	public void setArea() {
		area = new Area();
		for (int i = 0; i < getNumber(); i++) {
			area.add(getModel(i).getBaseArea());
		}
	}

	/**
	 * remove unused machines after killing
	 * 
	 */
	public void commitKill(int newNumber) {
		Figure model;
		for (int i = number - 1; i >= newNumber; i--) {
			model = (Figure) models.remove(i);
			BattleFieldModel.getInstance().removeFigure(model);
		}
		number = newNumber;
	}

	/**
	 * calculate machines cost
	 */
	public int cost() {
		return number * price;
	}

	/**
	 * create machine sprite
	 * 
	 */
	public void createsprite() {
		sprite = Sprite.getInstance(this);
	}

	/**
	 * delete sprite to free memory
	 * 
	 */
	public void freeSprite() {
		sprite = null;
	}

	/**
	 * @return the sprite
	 */
	public Sprite getSprite() {
		return sprite;
	}

	/**
	 * initialize machines at the beginning of a battle
	 * 
	 */
	public void initBattle() {
		area = new Area();
		models = new ArrayList();
	}

	/**
	 * create machines models
	 * 
	 */
	public void createModel() {
		Figure model;
		for (int i = 0; i < number; i++) {
			model = new Figure(this);
			model.init();
			models.add(model);
			BattleFieldModel.getInstance().addFigure(model);
		}

	}

	/**
	 * score needed to kill target
	 * 
	 * @param shooterWeapon
	 *            shooter weapon
	 * @param targetArmor
	 *            target armor
	 * @return score needed to kill target
	 */
	public int shootToKill(char shooterWeapon, char targetArmor) {
		if (shooterWeapon == 'g') {
			if (targetArmor == 'x') {
				return 4;
			} else if (targetArmor == 'h') {
				return 3;
			} else if (targetArmor == 'm') {
				return 3;
			} else {
				return 2;
			}
		} else /* if(shooterWeapon=='c') */{
			if (targetArmor == 'x') {
				return 4;
			} else if (targetArmor == 'h') {
				return 3;
			} else if (targetArmor == 'm') {
				return 3;
			} else {
				return 2;
			}
		}
	}

	/**
	 * @return Returns the length.
	 */
	public float getLength() {
		return length;
	}

	/**
	 * @param length
	 *            The length to set.
	 */
	public void setLength(float length) {
		this.length = length;
	}

	/**
	 * @return Returns the max.
	 */
	public int getMax() {
		return max;
	}

	/**
	 * @param max
	 *            The max to set.
	 */
	public void setMax(int max) {
		this.max = max;
	}

	/**
	 * @return Returns the min.
	 */
	public int getMin() {
		return min;
	}

	/**
	 * @param min
	 *            The min to set.
	 */
	public void setMin(int min) {
		this.min = min;
	}

	/**
	 * @return Returns the movement.
	 */
	public int getMovement() {
		return movement;
	}

	/**
	 * @param movement
	 *            The movement to set.
	 */
	public void setMovement(int movement) {
		this.movement = movement;
	}

	/**
	 * @return Returns the price.
	 */
	public int getPrice() {
		return price;
	}

	/**
	 * @param price
	 *            The price to set.
	 */
	public void setPrice(int price) {
		this.price = price;
	}

	/**
	 * @return Returns the strength.
	 */
	public int getStrength() {
		return strength;
	}

	/**
	 * @param strength
	 *            The strength to set.
	 */
	public void setStrength(int strength) {
		this.strength = strength;
	}

	/**
	 * @return Returns the width.
	 */
	public float getWidth() {
		return width;
	}

	/**
	 * @param width
	 *            The width to set.
	 */
	public void setWidth(float width) {
		this.width = width;
	}

	/**
	 * @return Returns the worth.
	 */
	public int getWorth() {
		return worth;
	}

	/**
	 * @param worth
	 *            The worth to set.
	 */
	public void setWorth(int worth) {
		this.worth = worth;
	}

	/**
	 * @return the medium Range
	 */
	public int getMidRange() {
		return midRange;
	}

	/**
	 * @return the shooting rows
	 */
	public int getShootingRows() {
		return shootingRows;
	}

	/**
	 * @return Returns the number.
	 */
	public int getNumber() {
		return number;
	}

	/**
	 * @param number
	 *            The number to set.
	 */
	public void setNumber(int number) {
		this.number = number;
	}

	/**
	 * get a machine model
	 * 
	 * @param index
	 *            model index
	 * @return the model
	 */
	public Figure getModel(int index) {
		return (Figure) models.get(index);
	}

	/**
	 * start shooting animation
	 * 
	 * @param target
	 *            shooting target
	 * @param shootingAngle
	 *            shooting angle
	 */
	public void animShoot(Entity target, double shootingAngle) {
		Warrior targetmodel;
		Figure model;

		for (int i = 0; i < getNumber(); i++) {
			model = getModel(i);

			if (target.isIndividual()) {
				targetmodel = ((Individual) target).getModel();
				model.prepareProjectile(new Projectile(model.getX(), model
						.getY(), targetmodel, projectileType));
			} else {
				Unit unit = (Unit) target;

				targetmodel = unit.nextTarget();

				List otherTargets = new ArrayList();
				for (int j = 0; j < getWorth() - 1; j++) {
					otherTargets.add(unit.nextTarget());
				}

				model.prepareProjectile(new Projectile(model.getX(), model
						.getY(), targetmodel, otherTargets, projectileType));
			}

			model.animshoot(shootingAngle);
		}
	}

	/**
	 * is this unit shooting
	 * 
	 * @return true if one of the machines is shooting
	 */
	public boolean isShooting() {
		for (int i = 0; i < getNumber(); i++) {
			if (getModel(i).getAction() == 's') {
				return true;
			}
		}
		return false;
	}

	/**
	 * @return long range
	 */
	public int getLongRange() {
		return longrange;
	}

	/**
	 * @return short range
	 */
	public int getShortRange() {
		return shortrange;
	}

	/**
	 * @return base area
	 */
	public Area getArea() {
		return area;
	}

	/**
	 * add sequences to sprite
	 */
	public void addSpriteSequences(Sprite sprite) {
		sprite.addSpritsequence("stand0", 4, 4, 4, 4);
		sprite.addSpritsequence("stand1", 5, 5, 5, 5);
		sprite.addSpritsequence("stand2", 6, 6, 6, 6);
		sprite.addSpritsequence("stand3", 7, 7, 7, 7);
		sprite.addSpritsequence("stand4", 0, 0, 0, 0);
		sprite.addSpritsequence("stand5", 1, 1, 1, 1);
		sprite.addSpritsequence("stand6", 2, 2, 2, 2);
		sprite.addSpritsequence("stand7", 3, 3, 3, 3);

		sprite.addSpritsequence("shoot0", 12, 20, 28, 36);
		sprite.addSpritsequence("shoot1", 13, 21, 29, 37);
		sprite.addSpritsequence("shoot2", 14, 22, 30, 38);
		sprite.addSpritsequence("shoot3", 15, 23, 31, 39);
		sprite.addSpritsequence("shoot4", 8, 16, 24, 32);
		sprite.addSpritsequence("shoot5", 9, 17, 25, 33);
		sprite.addSpritsequence("shoot6", 10, 18, 26, 34);
		sprite.addSpritsequence("shoot7", 11, 19, 27, 35);
	}

	/**
	 * get number of frames in sprite
	 */
	public int getNumberOfFrames() {
		return 40;
	}

	/**
	 * @return the sprite Number
	 */
	public int getSpriteID() {
		return spriteNumber;
	}

	/**
	 * clone this machine set
	 */
	public Object clone(Crew crew) {
		try {
			Object clone = super.clone();
			((WarMachine) clone).crew = crew;
			return clone;
		} catch (CloneNotSupportedException cnse) {
			cnse.printStackTrace();
			return null;
		}
	}

	/**
	 * get normal number of crew needed for one machine
	 * 
	 * @return normal number of crew needed for one machine
	 */
	public int getNormalCrewNumber() {
		return normalCrewNumber;
	}

	/**
	 * @return the initial Number
	 */
	public int getInitialNumber() {
		return initialNumber;
	}

	/**
	 * set initial number of machines
	 * 
	 * @param initialnumber
	 *            new initial number of machines
	 */
	public void setInitialNumber(int initialNumber) {
		this.initialNumber = initialNumber;
		this.number = initialNumber;
	}

	/**
	 * remove machines after routing
	 * 
	 */
	public void commitRout() {
		Figure model;
		for (int i = 0; i < number; i++) {
			model = getModel(i);
			BattleFieldModel.getInstance().removeFigure(model);
		}

	}

	/**
	 * get the army color
	 */
	public Color getColor() {
		return crew.getColor();
	}

	/**
	 * @return Returns the scale.
	 */
	public float getScale() {
		return crew.getScale();
	}

	/**
	 * get step size
	 */
	public int getStep() {
		return crew.getStep();
	}

	/**
	 * teleport machines to their destination
	 * 
	 */
	public void teleport() {

		for (int i = 0; i < number; i++) {
			getModel(i).teleport();
		}
	}

	/**
	 * organize machines positions
	 */
	public void organize() {
		int row = 0;
		int reste = getNumber();
		int xd, yd, line;
		float rowWidth;
		float rowOffset;
		for (int i = 0; i < getNumber();) {
			if (reste > getFront()) {
				line = getFront();
			} else {
				line = reste;
			}

			rowWidth = Math.max(crew.rowWidth(row), getWidth() * (float) line);
			rowOffset = 8 * rowWidth / (float) line;
			for (int j = 0; j < line; j++) {
				xd = (int) ((((float) j - (float) (line - 1) / 2f) * rowOffset) + crew
						.getX());

				if (crew.getFront() % 2 == 0) {
					xd -= (getWidth() * 8 / 2);
				}
				yd = (int) (crew.getLength() * 8 / 2
						+ (row * (getLength() + crew.getLength()) * 8) + crew
						.getY());
				getModel(i).moveTo(xd, yd);
				i++;
			}
			row++;
			reste -= getFront();
		}
	}

	/**
	 * get number of machines on the front line
	 * 
	 * @return number of machines on the front line
	 */
	public int getFront() {
		int result = getNumber() / crew.nbRows;
		if (getNumber() % crew.nbRows > 0) {
			result++;
		}

		result = Math.max(1, Math.min(result, number));
		return result;
	}

	/**
	 * start movement animation
	 * 
	 */
	public void animMovement() {
		for (int i = 0; i < number; i++) {
			getModel(i).animMovement();
		}

	}

}
