/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.campaign;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JPanel;

import fb.army.hierarchy.Army;
import fb.army.hierarchy.Force;
import fb.gui.panels.ArmyEditorPanel;
import fb.gui.widgets.ButtonListener;
import fb.gui.widgets.FBButton;
import fb.gui.widgets.FBToolBarButton;
import fb.main.FB;
import fb.util.FBColors;
import fb.util.PercentageLayout;
import fb.util.Resources;
/**
 * the army editor for campaign game
 * @author farfpeuf
 *
 */
public class CampaignArmyEditor extends JPanel {

	/**
	 * ok and cancel buttons
	 */
	private JButton ok ,cancel ;

	/**
	 * force for which a new army is risen
	 */
	private Force force;


/**
 * create new army editor panel for specified force
 * @param force the force for which a new army is risen
 */
	public CampaignArmyEditor(Force force) {
		
		buildFrame();
		this.force = force;
		ArmyEditorPanel.getInstance().setInitialBudget(force.getBudget());

	}

	/**
	 * build and assemble the panel elements
	 *
	 */
	private void buildFrame() {

		if (FB.getInstance().isMini()) {
			ok=new FBButton("ok");
			cancel = new FBButton("exit");
		}else{
			ok = new FBToolBarButton("save");
			cancel = new FBToolBarButton("exit");
		}
		
		ok.setToolTipText(Resources.getString("Ok"));
		cancel.setToolTipText(Resources.getString("Cancel"));

		setLayout(new PercentageLayout());
		setOpaque(false);
		JPanel buttonsPanel = new JPanel(new GridBagLayout());
		buttonsPanel.setBorder(BorderFactory
				.createLineBorder(FBColors.darkBorder));
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.weightx = 1;

		buttonsPanel.add(ok, gbc);
		buttonsPanel.add(cancel, gbc);
		add(buttonsPanel, new PercentageLayout.Constraint(10, 2, 6, 4));
		add(ArmyEditorPanel.getInstance(), new PercentageLayout.Constraint(5, 5,
				90, 90));

		setSize((int) (800 * FB.getXres()), (int) (600 * FB.getYres()));

		hookupEvents();

	}

	/**
	 * hook up buttons events
	 *
	 */
	private void hookupEvents() {


		
		// save
		ok.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				Army army = ArmyEditorPanel.getInstance().getArmy();
				army.resetRemainingMove();
				String message = army.check(force.getBudget());
				if (message.equals("")) {
					FB.getInstance().clearFrame();
					force.addArm(army);
					force.setBudget(force.getBudget()-army.getCost());
					CampaignStatePanel.getInstance().updateForceTab(force);
					FB.getInstance().dispalyCampaignGameFrame();
					Campaign.getInstance().setSelectedArmy(army);
				} else {
					FB.getInstance().popUpMessage(
							Resources.getString("cannotSaveArmy"), message);
				}
			}
		});

		// cancel
		cancel.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				FB.getInstance().clearFrame();
				FB.getInstance().dispalyCampaignGameFrame();
			}
		});
	} // hookupevents

}