/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.campaign;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import fb.army.entities.Entity;
import fb.army.hierarchy.Army;
import fb.army.hierarchy.Command;
import fb.gui.panels.MessageArea;
import fb.gui.panels.TreeTablePanel;
import fb.gui.widgets.ButtonListener;
import fb.gui.widgets.FBButton;
import fb.gui.widgets.armytreetable.TreeTableModelAdapter;
import fb.main.FB;
import fb.util.FBColors;
import fb.util.PercentageLayout;
import fb.util.Resources;

/**
 * The campaign game frame
 * 
 * @author farfpeuf
 * 
 */
public class CampaignGameFrame extends JPanel implements PropertyChangeListener {

	// maximal hexaagon radius
	private static final int MAX_RADIUS = 40;

	// minimal hexaagon radius
	private static final int MIN_RADIUS = 5;

	/**
	 * sigleton instance
	 */
	private static CampaignGameFrame instance;

	// buttons
	private FBButton nextPhase, zoomin, zoomout, quit, saveGame, rise;

	// mini map representation enabling to navigate quickly on the big map
	private MiniHexagonalMapPanel minimap;

	// sub-panels
	private JLabel northpanel, southpanel;

	// big map panel
	private CampaignMapPanel mapPanel;

	// current army tree panel
	private TreeTablePanel currentArmyPanel;

	/**
	 * get singleton instance
	 * 
	 * @return the singleton
	 */
	public static CampaignGameFrame getInstance() {
		if (null == instance) {
			instance = new CampaignGameFrame();
		}
		return instance;
	}

	/**
	 * free memory used by singleton instance
	 * 
	 */
	public static void freeInstance() {
		if (null != instance) {
			instance = null;
		}
	}

	/**
	 * calculate which button must be enabled
	 * 
	 */
	public void enableActions() {
		boolean waiting = Campaign.getInstance().isWaiting();
		nextPhase.setEnabled(!waiting);
		rise.setEnabled(!waiting
				&& Campaign.getInstance().currentForceControlsCapital()
				&& Campaign.getInstance().currentForce().isHandled()
				&& !Campaign.getInstance().currentForce().isAi());
		saveGame.setEnabled(!waiting);
	}

	/**
	 * constructor builds frame and assemble components
	 */
	public CampaignGameFrame() {

		setOpaque(false);
		setBackground(FBColors.background);

		minimap = new MiniHexagonalMapPanel();
		minimap.init(HexagonalMapModel.getInstance(), true);

		mapPanel = new CampaignMapPanel();
		if (!FB.getInstance().isMini()) {
			mapPanel.setRadius(30);
		}
		mapPanel.setModel(HexagonalMapModel.getInstance());
		northpanel = new JLabel(Resources.getLookNFeelImage("frise"));
		northpanel.setLayout(new PercentageLayout());

		northpanel.add(CampaignStatePanel.getInstance(),
				new PercentageLayout.Constraint(0, 0, 50, 100));

		currentArmyPanel = new TreeTablePanel(null, true);

		currentArmyPanel.getTreeTable().setColumnWidths(
				new int[] { 200, 80, 150 });
		currentArmyPanel.getTreeTable().getSelectionModel()
				.addListSelectionListener(new ListSelectionListener() {
					public void valueChanged(ListSelectionEvent e) {
						ListSelectionModel model = (ListSelectionModel) e
								.getSource();

						TreeTableModelAdapter ad = (TreeTableModelAdapter) currentArmyPanel
								.getTreeTable().getModel();
						Object leadNode = ad.nodeForRow(model
								.getLeadSelectionIndex());

						centerOn(leadNode);

					}
				});

		northpanel.add(currentArmyPanel, new PercentageLayout.Constraint(50, 0,
				50, 100));

		southpanel = new JLabel(Resources.getLookNFeelImage("frise"));

		JPanel textebuttons = new JPanel();
		JPanel zoombuttons = new JPanel();

		textebuttons.setLayout(new GridLayout(4, 1));
		zoombuttons.setLayout(new GridLayout(2, 1));

		nextPhase = new FBButton(Resources.getString("nextphasebutton"));
		zoomin = new FBButton(null, "zoomin", true);
		zoomout = new FBButton(null, "zoomout", true);
		quit = new FBButton(Resources.getString("Exittomainmenu"));
		saveGame = new FBButton(Resources.getString("saveCampaign"));
		rise = new FBButton(Resources.getString("riseNewArmy"));

		nextPhase.setToolTipText(Resources.getString("endphase.tt"));
		zoomin.setToolTipText(Resources.getString("zoomin.tt"));
		zoomout.setToolTipText(Resources.getString("zoomout.tt"));
		quit.setToolTipText(Resources.getString("backtomenu.tt"));
		saveGame.setToolTipText(Resources.getString("saveCampaign.tt"));
		rise.setToolTipText(Resources.getString("rise.tt"));

		zoombuttons.add(zoomin);
		zoombuttons.add(zoomout);

		textebuttons.add(rise);
		textebuttons.add(nextPhase);
		textebuttons.add(saveGame);
		textebuttons.add(quit);

		// build minimappanel

		JPanel miniMapPanel = new JPanel();
		miniMapPanel.setLayout(new GridBagLayout());
		miniMapPanel.add(minimap, new GridBagConstraints());

		southpanel.setLayout(new PercentageLayout());

		southpanel.add(zoombuttons, new PercentageLayout.Constraint(0, 0, 6, 100));
		southpanel
				.add(miniMapPanel, new PercentageLayout.Constraint(7, 1, 18, 98));
		southpanel.add(textebuttons, new PercentageLayout.Constraint(26, 0, 12,
				100));

		if (FB.getInstance().isMini()) {
			setLayout(new PercentageLayout());
			// add(northpanel, new PercentLayout.Constraint(0, 0, 100, 40));
			// add(mapPanel, new PercentLayout.Constraint(0, 40, 100, 40));
			add(mapPanel, new PercentageLayout.Constraint(0, 0, 100, 80));
			add(southpanel, new PercentageLayout.Constraint(0, 80, 100, 20));
		} else {
			setLayout(new GridBagLayout());
			GridBagConstraints gbc = new GridBagConstraints();
			gbc.gridx = 0;
			gbc.gridy = 0;
			gbc.weightx = 1;
			gbc.fill = GridBagConstraints.BOTH;
			add(northpanel, gbc);
			gbc.weighty = 1;
			gbc.gridy++;
			add(mapPanel, gbc);
			gbc.weighty = 0;
			gbc.gridy++;
			add(southpanel, gbc);
		}
		validate();

		hookupEvents();
	}

	/**
	 * hookup frame events
	 * 
	 */
	private void hookupEvents() {

		minimap.addMouseListener(new MouseAdapter() {
			public void mouseReleased(MouseEvent e) {
				Point hexaPoint = minimap.getHexaCoord(e.getPoint());
				mapPanel.centerOn(hexaPoint);
			} // mouse clicked
		});
		mapPanel.addMouseListener(new MouseAdapter() {
			public void mouseReleased(final MouseEvent e) {
				if (e.getButton() == MouseEvent.BUTTON1) {
					Thread clickThread = new Thread("player click") {
						public void run() {
							mapPanel.click(e.getPoint());
						}
					};
					clickThread.start();
				} else {
					Point hexaClick = mapPanel.getHexaCoord(e.getPoint());
					Army arm = Campaign.getInstance().getArmy(hexaClick);
					currentArmyPanel.setRoot(arm);
				}
			} // mouse clicked
		});

		mapPanel.addMouseWheelListener(new MouseWheelListener() {
			public void mouseWheelMoved(MouseWheelEvent e) {
				int roll = e.getWheelRotation();

				int radius = mapPanel.getRadius() - (roll * 3);
				if (radius >= MAX_RADIUS) {
					radius = MAX_RADIUS;
					zoomin.setEnabled(false);
				} else {
					zoomin.setEnabled(true);
				}
				if (radius <= MIN_RADIUS) {
					radius = MIN_RADIUS;
					zoomout.setEnabled(false);
				} else {
					zoomout.setEnabled(true);
				}

				mapPanel.setRadius(radius);

			} // mouse wheel moved
		});

		nextPhase.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				Campaign.getInstance().changeForce();
			}
		});
		rise.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				FB.getInstance().displayCampaignArmyEditorFrame(
						Campaign.getInstance().currentForce());
			}
		});

		quit.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				MessageArea.getInstance().clear();
				FB.getInstance().displayMainMenuFrame();
				FB.getInstance().autoSaveCampaign();
				stop();
				Campaign.getInstance().stop();
			}
		});
		saveGame.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				saveGame.setEnabled(false);
				FB.getInstance().saveCampaign(false);
				saveGame.setEnabled(true);
			}
		});
		zoomin.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {

				int radius = mapPanel.getRadius() + 3;
				if (radius >= MAX_RADIUS) {
					radius = MAX_RADIUS;
					zoomin.setEnabled(false);
				}
				mapPanel.setRadius(radius);
				zoomout.setEnabled(true);
			}
		});
		zoomout.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {

				int radius = mapPanel.getRadius() - 3;
				if (radius <= MIN_RADIUS) {
					radius = MIN_RADIUS;
					zoomout.setEnabled(false);
				}
				mapPanel.setRadius(radius);
				zoomin.setEnabled(true);
			}
		});

	} // hookupevents

	/**
	 * start frame animation
	 */
	public void start() {
		mapPanel.start();
		Campaign.getInstance().addPropertyChangeListener(
				Campaign.WAITING_PROPERTY, this);
		Campaign.getInstance().addPropertyChangeListener(
				Campaign.CURRENT_FORCE_PROPERTY, this);
		Campaign.getInstance().addPropertyChangeListener(
				Campaign.SELECTED_ARM_PROPERTY, this);
		enableActions();
	}

	/**
	 * stop frame animation
	 */
	public void stop() {
		Campaign.getInstance().removePropertyChangeListener(
				Campaign.WAITING_PROPERTY, this);
		Campaign.getInstance().removePropertyChangeListener(
				Campaign.CURRENT_FORCE_PROPERTY, this);
		mapPanel.stop();
	}

	/**
	 * center map view on specified army node
	 * 
	 * @param node
	 */
	public void centerOn(Object node) {
		Army arm = null;
		if (node instanceof Army) {
			arm = (Army) node;
		} else if (node instanceof Command) {
			arm = ((Command) node).getArmy();
		} else if (node instanceof Entity) {
			arm = ((Entity) node).getArmy();
		}
		if (arm != null) {
			mapPanel.centerOn(arm.getPosition());
		}
	}

	/**
	 * handle property Changes
	 */
	public void propertyChange(PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals(Campaign.WAITING_PROPERTY)) {
			enableActions();
		} else if (evt.getPropertyName()
				.equals(Campaign.CURRENT_FORCE_PROPERTY)) {
			enableActions();
		} else if (evt.getPropertyName().equals(Campaign.SELECTED_ARM_PROPERTY)) {
			Army arm = (Army) evt.getNewValue();
			if (arm != null) {
				currentArmyPanel.setRoot(arm);
			}
		}

	}

}
