/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.frames.popups;

import java.awt.GridLayout;

import javax.swing.JComboBox;
import javax.swing.JInternalFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import fb.gui.widgets.ButtonListener;
import fb.gui.widgets.FBButton;
import fb.gui.widgets.FBComboBox;
import fb.main.FB;
import fb.util.Resources;

/**
 * a popup frame containing a combo box to chosse a value
 * 
 * @author Farfpeuf
 */
public class ComboPopup extends JInternalFrame {
	/**
	 * the combo selected value
	 */
	private String stringToReturn = null;

	/**
	 * the combo box
	 */
	private JComboBox combo;

	/**
	 * constructor build and assemble elements
	 * 
	 * @param elements
	 *            the available values
	 * @param key
	 *            the title key
	 * @param labelKey
	 *            the message key
	 * @param editableCombo
	 *            can a free value be entered
	 */
	public ComboPopup(String[] elements, String key, String labelKey,
			boolean editableCombo) {
		super(Resources.getString(key), false, false, false, false);

		if (elements.length != 0) {
			combo = new FBComboBox(elements);
			combo.setEditable(editableCombo);
			combo.setSelectedIndex(0);
			FBButton ok = new FBButton(Resources.getString("OK"));
			FBButton cancel = new FBButton(Resources.getString("Cancel"));

			JLabel nameLabel = new JLabel(Resources.getString(labelKey));

			nameLabel.setHorizontalAlignment(SwingConstants.CENTER);

			JPanel cont = new JPanel();
			cont.setLayout(new GridLayout(2, 2));

			cont.add(nameLabel);
			cont.add(combo);

			cont.add(ok);
			cont.add(cancel);

			ok.addActionListener(new ButtonListener() {
				public void threadedActionPerformed() {

					if (combo.getSelectedItem() != null) {

						stringToReturn = (String) combo.getSelectedItem();
						closeFrame();

					}
				}
			});

			cancel.addActionListener(new ButtonListener() {
				public void threadedActionPerformed() {
					closeFrame();
				}
			});

			getContentPane().add(cont);

			setBounds((int) (300 * FB.getXres()), (int) (100 * FB.getYres()),
					(int) (Math.max(80, 200 * FB.getXres())), (int) (Math.max(
							80, 60 * FB.getYres())));

			validate();

		}
	}

	/**
	 * wait until a value is selected
	 * 
	 * @return the selected value
	 */
	public synchronized String getString() {
		toFront();
		show();
		try {
			wait();
		} catch (InterruptedException e) {
			System.out.println("interrupted");
		}
		return stringToReturn;
	}

	/**
	 * cloase the popup
	 * 
	 */
	private synchronized void closeFrame() {
		dispose();
		try {
			notify();

		} catch (IllegalMonitorStateException e) {
			System.out.println("illegal monitor state");
			e.printStackTrace();
		}
	}

}
