/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.panels;

import java.awt.Color;
import java.util.Hashtable;

import javax.swing.JPanel;
import javax.swing.JTextField;

import fb.gui.widgets.ButtonListener;
import fb.gui.widgets.FBButton;
import fb.gui.widgets.FBTextArea;
import fb.network.Client;
import fb.network.MessagesConstants;
import fb.util.PercentageLayout;
import fb.util.Resources;

/**
 * panel used to chat with other players on the network
 * 
 * @author Farfpeuf
 */
public class ChatArea extends JPanel {
	/**
	 * singleton instance
	 */
	private static ChatArea instance;

	/**
	 * clients id/name map
	 */
	private Hashtable clientNames = new Hashtable();

	/**
	 * area to display messages
	 */
	private FBTextArea textArea = new FBTextArea();

	/**
	 * button to send new massage
	 */
	private FBButton chatButton = new FBButton(Resources.getString("Chat"));

	/**
	 * field to enter new message
	 */
	private JTextField chatField = new JTextField("");

	/**
	 * get singleton instance
	 * 
	 * @return the singleton
	 */
	public static synchronized ChatArea getInstance() {
		if (null == instance) {
			instance = new ChatArea();
		}
		return instance;
	}

	/**
	 * constructor 
	 * register events and assemble components
	 */
	private ChatArea() {

		chatButton.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				chat();
			}
		});
		chatField.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				chat();
			}
		});
		setLayout(new PercentageLayout());
		add(textArea, new PercentageLayout.Constraint(0, 0, 100, 75));
		add(chatField, new PercentageLayout.Constraint(0, 75, 80, 25));
		add(chatButton, new PercentageLayout.Constraint(80, 75, 20, 25));

	}

	/**
	 * send message in textfield
	 * 
	 */
	private void chat() {
		Client.getInstance().send(
				MessagesConstants.CHAT + Client.getInstance().getClientId()
						+ "#1#" + chatField.getText());
		chatField.setText("");
	}

	/**
	 * add a client
	 * 
	 * @param clientId
	 *            the client id
	 * @param clientName
	 *            the client name
	 */
	public void addClientName(int clientId, String clientName) {
		Integer id = new Integer(clientId);
		if (null == clientNames.get(id)) {
			clientNames.put(id, clientName);
		}
	}

	/**
	 * add a message in the message area
	 * 
	 * @param clientId
	 *            the client id
	 * @param text
	 *            the message
	 * @param color
	 *            the message color
	 */
	public void writeText(int clientId, String text, Color color) {
		String clientName = (String) clientNames.get(new Integer(clientId));
		if (clientName != null) {
			textArea.appendText("[" + clientName + "] " + text, color);
		} else {
			writeText(text, color);
		}
	}

	/**
	 * add a message in the message area
	 * 
	 * @param text
	 *            the message
	 * @param color
	 *            the message color
	 */
	public void writeText(String text, Color color) {
		textArea.appendText(text, color);
	}

}
