/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.panels;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.ToolTipManager;

import fb.main.Battle;
import fb.phases.Phase;
import fb.util.FBColors;
import fb.util.Resources;

/**
 * panel to display current phase icon and quick help
 * 
 * @author coheris
 * 
 */
public class PhasePanel extends JPanel {

	/**
	 * the phase icon panel
	 */
	private JLabel iconlabel;

	/**
	 * the phase quick help text area
	 */
	private JLabel textArea;

	/**
	 * panel scroller
	 */
	private JScrollPane scroller;

	/**
	 * set current phase
	 * 
	 * @param phase
	 *            the new phase
	 */
	public void setPhase(Phase phase) {

		String text = phase.getTooltip();
		scroller.setVisible(text != null
				&& ToolTipManager.sharedInstance().isEnabled());
		textArea.setText(text);

		iconlabel.setIcon(Resources.getPhaseIcon(Battle.getLoop()
				.getCurrentPhase().getIconNumber()));
		iconlabel.setToolTipText(Resources.getString(phase.getName()));

		validate();
		repaint();
	}

	/**
	 * constructor build and assemble panel elements
	 */
	public PhasePanel() {

		setLayout(new GridBagLayout());

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 0;
		gbc.insets = new Insets(0, 10, 0, 0);

		iconlabel = new JLabel();
		textArea = new JLabel();

		scroller = new JScrollPane(textArea,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_ALWAYS);

		textArea.setForeground(FBColors.standardText);
		textArea.setBackground(FBColors.background);
		textArea.setOpaque(true);
		gbc.anchor = GridBagConstraints.WEST;
		gbc.weightx = 0.01;
		gbc.weighty = 0.01;
		add(iconlabel, gbc);
		gbc.gridx++;
		gbc.weightx = 1;
		gbc.weighty = 1;
		gbc.fill = GridBagConstraints.BOTH;
		add(scroller, gbc);

		setSize(630, 120);
		setOpaque(false);
		validate();

	}

}
