/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.panels;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.io.File;
import java.util.List;
import java.util.Vector;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import fb.army.entities.Entity;
import fb.gui.widgets.ButtonListener;
import fb.gui.widgets.FBButton;
import fb.gui.widgets.FBComboBox;
import fb.gui.widgets.FBTextArea;
import fb.gui.widgets.SlideValueChooser;
import fb.main.Battle;
import fb.phases.PlayerMagicPhase;
import fb.spells.BasicSpell;
import fb.spells.TargetSpell;
import fb.spells.Telepathy;
import fb.spells.ValuedSpell;
import fb.util.FBColors;
import fb.util.PercentageLayout;
import fb.util.Resources;
/**
 * panel to display spell characteristics and to choose spell options
 * @author farfpeuf
 *
 */
public class SpellPanel extends JPanel {
	/**
	 * the spell help text
	 */
	private String helptext;
	/**
	 * sub panels
	 */
	private JLabel iconlabel, title, targetLabel, orderLabel, averageCost, minCost,
			maxCost,noTargetLabel;
	/**
	 * help text area
	 */
	private FBTextArea textArea;
	/**
	 * combo to choose target (target spells) and order (telepathy spell)
	 */
	private FBComboBox targetCombo, orderCombo;
	/**
	 * action buttons
	 */
	private FBButton castButton, backButton;
	/**
	 * current spell
	 */
	private BasicSpell spell;
	/**
	 * spell value chooser (valued spells only)
	 */
	private SlideValueChooser svc;
	/**
	 * set the spell to cast
	 * @param spell the spell
	 */
	public void setSpell(BasicSpell spell) {
		this.spell = spell;
		title.setText(Resources.getString(spell.getName()));
		textArea.setText(Resources.getString(spell.getName() + "help"), FBColors.standardText);
		iconlabel.setIcon(Resources.getScaledImage("buttons" + File.separator
				+ spell.getName() + ".gif", 50, 50));
		targetLabel.setVisible(spell instanceof TargetSpell);
		targetCombo.setVisible(spell instanceof TargetSpell);
		noTargetLabel.setVisible(targetCombo.isVisible());
		orderCombo.setVisible(spell instanceof Telepathy);
		orderLabel.setVisible(spell instanceof Telepathy);
		svc.setVisible(spell instanceof ValuedSpell);
		svc.setVisible(spell instanceof ValuedSpell);
		if (spell instanceof ValuedSpell) {
			ValuedSpell vs = (ValuedSpell) spell;
			svc.setText(vs.valueName());
			svc.setValues(vs.minValue(), vs.maxValue(), vs.currentValue());
		}
		if (!(spell instanceof TargetSpell)) {
			showCosts();
		}
	}
	/**
	 * constructor build and assemble panel elements
	 */
	public SpellPanel() {
		JPanel spellCaracs = new JPanel();
		JPanel costPanel = new JPanel();
		JPanel paramPanel = new JPanel();
		setLayout(new PercentageLayout());
		spellCaracs.setLayout(new GridBagLayout());
		paramPanel.setLayout(new GridBagLayout());
		costPanel.setLayout(new GridLayout(3, 1));
		castButton = new FBButton(Resources.getString("cast"));
		backButton = new FBButton(Resources.getString("backToSpellChoice"));
		targetLabel = new JLabel(Resources.getString("target"));
		orderLabel = new JLabel(Resources.getString("newOrders"));
		svc = new SlideValueChooser("", true);
		targetCombo = new FBComboBox();
		String[] orders = new String[]{Resources.getString("attack"),
				Resources.getString("oppose"), Resources.getString("hold")};
		orderCombo = new FBComboBox(orders);
		textArea = new FBTextArea(false);
		iconlabel = new JLabel();
		averageCost = new JLabel();
		minCost = new JLabel();
		maxCost = new JLabel();
		noTargetLabel = new JLabel(Resources.getString("notarget"),0);
		title = new JLabel("", 0);
		title.setOpaque(true);
		spellCaracs.setOpaque(true);
		targetCombo.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				targetComboChanged(e);
			}
		});
		orderCombo.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				orderComboChanged(e);
			}
		});
		svc.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				((ValuedSpell) spell).setValue(((JSlider)e.getSource()).getValue());
				showCosts();
			}
		});
		castButton.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				castButton.setEnabled(false);
				((PlayerMagicPhase) Battle.getLoop().currentSynchronizedPhase).castSpell();
				if (!(spell instanceof TargetSpell)) {
					castButton
							.setEnabled(spell.getCaster().getMagicPoints() >= spell
									.minCost());
				}
			}
		});
		backButton.addActionListener(new ButtonListener() {
			public void threadedActionPerformed() {
				((PlayerMagicPhase) Battle.getLoop().currentSynchronizedPhase)
						.initSpellSelection();
			}
		});
		add(iconlabel, new PercentageLayout.Constraint(1, 30, 10, 40));
		add(title, new PercentageLayout.Constraint(12, 0, 88, 13));
		add(spellCaracs, new PercentageLayout.Constraint(12, 13, 88, 67));
		add(castButton, new PercentageLayout.Constraint(12, 80, 44, 20));
		add(backButton, new PercentageLayout.Constraint(56, 80, 44, 20));
		costPanel.add(averageCost);
		costPanel.add(minCost);
		costPanel.add(maxCost);
		spellCaracs.add(textArea, new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
				GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(
						5, 5, 5, 5), 0, 0));
		spellCaracs.add(costPanel, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
						5, 5, 5, 5), 0, 0));
		spellCaracs.add(paramPanel, new GridBagConstraints(0, 1, 2, 1, 1.0,
				0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
				new Insets(0, 0, 0, 0), 0, 0));
		paramPanel.add(targetLabel, new GridBagConstraints(0, 0, 1, 1, 0.0,
				0.0, GridBagConstraints.WEST, GridBagConstraints.NONE,
				new Insets(5, 5, 5, 5), 0, 0));
		paramPanel.add(targetCombo, new GridBagConstraints(1, 0, 1, 1, 1.0,
				0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
				new Insets(5, 5, 5, 5), 0, 0));
		paramPanel.add(noTargetLabel, new GridBagConstraints(1, 0, 1, 1, 1.0,
				0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
				new Insets(5, 5, 5, 5), 0, 0));
		paramPanel.add(svc, new GridBagConstraints(2, 0, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5,
						5, 5, 5), 0, 0));
		paramPanel.add(orderLabel, new GridBagConstraints(3, 0, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(5,
						5, 5, 5), 0, 0));
		paramPanel.add(orderCombo, new GridBagConstraints(4, 0, 1, 1, 0.0, 0.0,
				GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL,
				new Insets(5, 5, 5, 5), 0, 0));
		setSize(550, 120);
		setOpaque(false);
		validate();
	}
	/**
	 * handle change of target
	 * @param e the change event
	 */
	private void targetComboChanged(ItemEvent e) {
		if (e.getStateChange() == ItemEvent.SELECTED) {
			((TargetSpell) spell).setTarget((Entity) e.getItem());
			showCosts();
		}
	}
	/**
	 * handle change of order
	 * @param e the change event
	 */
	private void orderComboChanged(ItemEvent e) {
		if (e.getStateChange() == ItemEvent.SELECTED) {
			char newOrder = 'a';
			switch (orderCombo.getSelectedIndex()) {
				case 0 :
					newOrder = 'a';
				case 1 :
					newOrder = 'o';
				case 2 :
					newOrder = 'h';
			}
			((Telepathy) spell).setNeworder(newOrder);
		}
	}
	/**
	 * display spell costs
	 *
	 */
	private void showCosts() {
		if (spell instanceof TargetSpell) {
			showCosts((Entity) targetCombo.getSelectedItem());
		} else {
			averageCost.setText(Resources.getString("averagecost")
					+ spell.averageCost());
			minCost.setText(Resources.getString("mincost") + spell.minCost());
			maxCost.setText(Resources.getString("maxcost") + spell.maxCost());
			castButton.setEnabled(spell.getCaster().getMagicPoints() >= spell
					.minCost());
		}
	}
	/**
	 * display spell costs for specified target entity
	 * @param e the target entity
	 */
	private void showCosts(Entity e) {
		if (e != null) {
			averageCost.setText(Resources.getString("averagecost")
					+ ((TargetSpell) spell).averageCost(spell.getCaster(), e));
			minCost.setText(Resources.getString("mincost")
					+ ((TargetSpell) spell).minCost(spell.getCaster(), e));
			maxCost.setText(Resources.getString("maxcost")
					+ ((TargetSpell) spell).maxCost(spell.getCaster(), e));
			castButton.setEnabled(true);
		} else {
			averageCost.setText(Resources.getString("averagecost") + "-");
			minCost.setText(Resources.getString("mincost") + "-");
			maxCost.setText(Resources.getString("maxcost") + "-");
			castButton.setEnabled(false);
		}
	}
	/**
	 * change current spell target
	 * @param target the new target
	 */
	public void setTarget(Entity target) {
		targetCombo.setSelectedItem(target);
	}
	/**
	 * set available targets
	 * @param availableTargets available targets
	 */
	public void setAvailableTargets(List availableTargets) {
		Vector targets = new Vector();
		targets.addAll(availableTargets);
		targetCombo.setModel(new DefaultComboBoxModel(targets));
		if (!availableTargets.isEmpty()) {
			((TargetSpell) spell).setTarget((Entity) targetCombo.getItemAt(0));
			if (spell instanceof ValuedSpell) {
				ValuedSpell vs = (ValuedSpell) spell;
				svc.setValues(vs.minValue(), vs.maxValue(), vs.currentValue());
			}
		} else {
			targetCombo.setSelectedIndex(-1);
			targetCombo.setVisible(false);
			noTargetLabel.setVisible(true);
		}
		showCosts();
	}
}