/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.panels;

import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import fb.army.entities.Unit;
import fb.gui.widgets.FBCheckBox;
import fb.gui.widgets.SlideValueChooser;
import fb.util.FBColors;
import fb.util.PercentageLayout;
import fb.util.Resources;

/**
 * additionnal panel to display unit recruitment options
 * 
 * @author farfpeuf
 * 
 */
public class UnitRecruitPanel extends RecruitPanel {

	/**
	 * current unit
	 */
	private Unit unit;

	/**
	 * widget to choose entity size
	 */
	private SlideValueChooser sizeChooser;

	/**
	 * checkboxes for boolean options
	 */
	private FBCheckBox standardBearerBox, championBox, upgradeLevelBox,
			musicianBox;

	/**
	 * unit cost label
	 */
	private JLabel costLabel;

	/**
	 * constructor build and assemble panel elements
	 */
	public UnitRecruitPanel() {

		JPanel comboPanel = new JPanel();

		JPanel cadrePanel = new JPanel();
		setBorder(BorderFactory.createLineBorder(FBColors.lightBorder));

		cadrePanel.setLayout(new PercentageLayout());
		comboPanel.setLayout(new GridLayout(4, 1));
		sizeChooser = new SlideValueChooser(" " + Resources.getString("Size")
				+ " : ");
		musicianBox = new FBCheckBox(" " + Resources.getString("Musician"));
		standardBearerBox = new FBCheckBox(" "
				+ Resources.getString("Standardbearer"));
		championBox = new FBCheckBox(" " + Resources.getString("Champion"));
		upgradeLevelBox = new FBCheckBox(" "
				+ Resources.getString("Upgradelevel"));
		costLabel = new JLabel();
		costLabel.setHorizontalAlignment(0);

		comboPanel.add(championBox);
		comboPanel.add(standardBearerBox);
		comboPanel.add(musicianBox);
		comboPanel.add(upgradeLevelBox);

		cadrePanel
				.add(sizeChooser, new PercentageLayout.Constraint(10, 0, 40, 70));
		cadrePanel.add(costLabel, new PercentageLayout.Constraint(0, 70, 50, 30));
		cadrePanel
				.add(comboPanel, new PercentageLayout.Constraint(50, 0, 50, 100));

		setLayout(new GridLayout(1, 1));
		add(cadrePanel);
		validate();

		sizeChooser.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				unit.setInitialNumber(((JSlider) e.getSource()).getValue());
				fireEntityChanged();
				updatecost();
			}
		});

		musicianBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				if (unit.getMusicians() == 0) {
					unit.setMusicians(1);
				} else {
					unit.setMusicians(0);
				}
				fireEntityChanged();
				updatecost();
			}
		});

		championBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				unit.setHasChampion(!unit.hasChampion());
				fireEntityChanged();
				updatecost();
			}
		});

		standardBearerBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				unit.setHasStandardBearer(!unit.hasStandardBearer());
				fireEntityChanged();
				updatecost();
			}
		});

		upgradeLevelBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent ev) {
				unit.setUpgraded(!unit.isUpgraded());
				if (unit.isUpgraded()) {
					unit.upgradeLevel();
				} else {
					unit.downgradeLevel();
				}
				fireEntityChanged();
				updatecost();
			}
		});
		championBox.setToolTipText(Resources.getString("champion.tt"));
		musicianBox.setToolTipText(Resources.getString("musician.tt"));
		standardBearerBox.setToolTipText(Resources.getString("standard.tt"));
		upgradeLevelBox.setToolTipText(Resources.getString("upgraded.tt"));
	}

	/**
	 * change the unit
	 * 
	 * @param unit
	 *            the new unit
	 */
	public void setUnit(Unit unit) {
		this.unit = unit;
		upgradeLevelBox.setEnabled(unit.isUpgradable());
		sizeChooser.setValues(unit.getMin(), unit.getMax(), unit.getNumber());
		musicianBox.setSelected(unit.getMusicians() == 1);
		standardBearerBox.setSelected(unit.hasStandardBearer());
		championBox.setSelected(unit.hasChampion());
		upgradeLevelBox.setSelected(unit.isUpgraded());
		costLabel.setText(" " + Resources.getString("Cost") + " : "
				+ unit.getCost());

	}

	/**
	 * update cost label
	 * 
	 */
	public void updatecost() {
		costLabel.setText(Resources.getString("Cost") + " : " + unit.getCost());
	}

}
