/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.widgets;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import javax.swing.ImageIcon;
import javax.swing.JComponent;

import fb.util.FBColors;
import fb.util.Resources;

/**
 * a check box to choose if player is human or artificial intelligence
 * 
 * @author farfpeuf
 * 
 */
public class AICheckBox extends JComponent implements MouseListener,
		MouseMotionListener {
	/**
	 * checked and uncheked icon
	 */
	private static ImageIcon checked = Resources.getLookNFeelImage("iacheckedbox"),
			unchecked = Resources.getLookNFeelImage("iauncheckedbox"),
			pressed = Resources.getLookNFeelImage("iapressedbox");

	/**
	 * current checkbox icon
	 */
	private ImageIcon currentIcon;

	/**
	 * current value (checked or unchecked)
	 */
	public boolean value = false;

	/**
	 * action listener
	 */
	protected ActionListener actionlistener = null;

	/**
	 * label color
	 */
	private Color textColor = FBColors.buttonText;

	/**
	 * constructor
	 * 
	 * @param init
	 *            initial value
	 */
	public AICheckBox(boolean init) {
		super();
		value = init;

		if (value) {
			setIcon(checked);
		} else {
			setIcon(unchecked);
		}
		addMouseListener(this);
		addMouseMotionListener(this);
	}

	/**
	 * change displayed icon
	 * 
	 * @param icon
	 *            the new icon
	 */
	private void setIcon(ImageIcon icon) {
		currentIcon = icon;
		repaint();
	}

	/**
	 * overidden to draw icon
	 */
	public void paintComponent(Graphics g) {
		int min = getWidth();
		if (getHeight() < min) {
			min = getHeight();
		}
		g.drawImage(currentIcon.getImage(), (getWidth() - min) / 2,
				(getHeight() - min) / 2, min, min, null);

	}

	// overidden to remove border

	public void paintBorder(Graphics g) {
	}

	// mouse events managment
	public void mouseEntered(MouseEvent event) {
	}

	public void mouseExited(MouseEvent event) {
	}

	public void mouseMoved(MouseEvent event) {
	}

	public void mouseDragged(MouseEvent event) {
	}

	public void mouseClicked(MouseEvent event) {
	}

	public void mousePressed(MouseEvent event) {
		if (isEnabled()) {
			setIcon(pressed);
		}
	}

	public void mouseReleased(MouseEvent event) {
		if (isEnabled()) {
			clic();
		}
	}

	/**
	 * set the action listener
	 * 
	 * @param al
	 *            the action listener
	 */
	public void setActionListener(ActionListener al) {
		actionlistener = al;
	}

	/**
	 * handle mouse clic
	 * 
	 */
	protected void clic() {
		value = !value;
		if (value) {
			setIcon(checked);
		} else {
			setIcon(unchecked);
		}
		if (actionlistener != null) {
			actionlistener.actionPerformed(new ActionEvent(this, 0, "clic"));
		}
	}

	/**
	 * Returns the value.
	 * 
	 * @return boolean
	 */
	public boolean isValue() {
		return value;
	}

	/**
	 * Sets the value.
	 * 
	 * @param value
	 *            The value to set
	 */
	public void setValue(boolean value) {
		if (this.value != value) {
			this.value = value;
			if (value) {
				setIcon(checked);
			} else {
				setIcon(unchecked);
			}
			if (actionlistener != null) {
				actionlistener.actionPerformed(new ActionEvent(this, 1,
						"changed"));
			}
		}
	}

	/**
	 * Sets the value.
	 * 
	 * @param value
	 *            The value to set
	 */
	public void setValue(Boolean value) {
		setValue(value.booleanValue());
	}
}