/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.widgets;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Insets;
import java.awt.LayoutManager;

import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;

/**
 * renderer used to display a combo box inside a table
 * 
 * @author farfpeuf
 * 
 */
public class ChoiceRenderer extends JPanel implements TableCellRenderer {
	/**
	 * renderer for the combo box selected value
	 */
	private StringRenderer stringRenderer = new StringRenderer();

	/**
	 * renderer for the combo box arrow button
	 */
	private FBButton arrowButton = new FBButton("", "combo");

	/**
	 * TableCellRenderer implementation
	 */
	public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int column) {
		stringRenderer.setText((String) value);
		return this;
	}

	/**
	 * constructor build and assemble elements
	 */
	public ChoiceRenderer() {
		super();
		setLayout(new ChoiceRendererLayout());
		add(stringRenderer);
		add(arrowButton);
	}

	/**
	 * special layout manager that handles the arrow button
	 * 
	 * @author farfpeuf
	 * 
	 */
	public class ChoiceRendererLayout implements LayoutManager {
		/**
		 * Lays out the specified container.
		 * 
		 * @param parent
		 *            the container to be laid out
		 */
		public void layoutContainer(Container parent) {
			ChoiceRenderer cb = (ChoiceRenderer) parent;
			int width = cb.getWidth();
			int height = cb.getHeight();
			Insets insets = getInsets();
			int buttonSize = height - (insets.top + insets.bottom);
			if (arrowButton != null) {
				arrowButton.setBounds(width - (insets.right + buttonSize),
						insets.top, buttonSize, buttonSize);
			}
			if (stringRenderer != null) {
				stringRenderer.setBounds(insets.left, insets.top, width
						- (insets.left + insets.right + buttonSize), height
						- (insets.top + insets.bottom));
			}
		}

		// LayoutManager implementation
		public void addLayoutComponent(String name, Component comp) {
		}

		public void removeLayoutComponent(Component comp) {
		}

		public Dimension preferredLayoutSize(Container parent) {
			return parent.getPreferredSize();
		}

		public Dimension minimumLayoutSize(Container parent) {
			return parent.getMinimumSize();
		}
	}
}