/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.widgets;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;

import fb.army.entities.Entity;
import fb.army.entities.Unit;
import fb.util.Resources;

/**
 * panel used to display an entity state
 * 
 * @author Farfpeuf
 */
public class EntityStateRenderer extends JPanel {
	/**
	 * state icons
	 */
	private static ImageIcon chiefIcon = Resources.getMinicon("chief"),
			championIcon = Resources.getMinicon("champion"),
			bloodLustIcon = Resources.getMinicon("bloodlust"),
			shakenIcon = Resources.getMinicon("shaken"),
			annihilatedIcon = Resources.getMinicon("skull"),
			routedIcon = Resources.getMinicon("routed"),
			disorganizedIcon = Resources.getMinicon("disorganized"),
			standardIcon = Resources.getMinicon("standard"),
			musicianIcon = Resources.getMinicon("musician"),
			upgradedIcon = Resources.getMinicon("upgraded"),
			chargeIcon = Resources.getMinicon("charge"),
			manoeuvreIcon = Resources.getMinicon("manoeuvre"),
			holdIcon = Resources.getMinicon("defend");

	/**
	 * panel cells
	 */
	private JLabel cellChief = new JLabel(chiefIcon),
			cellChampion = new JLabel(championIcon), cellStandard = new JLabel(
					standardIcon), cellMusician = new JLabel(musicianIcon),
			cellUpgraded = new JLabel(upgradedIcon), cellOrder = new JLabel(),
			cellState = new JLabel(), cellDisorganized = new JLabel(
					disorganizedIcon);

	/**
	 * standard gidbag constraint
	 */
	private GridBagConstraints constraint = new GridBagConstraints(-1, 0, 1, 1,
			0, 0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(
					0, 0, 0, 0), 0, 0);

	/**
	 * renderer used in army editor or in battle information table ?
	 */
	private boolean battleMode;

	/**
	 * set the panel entity
	 * 
	 * @param e
	 *            the entity
	 */
	public void setEntity(Entity e) {
		boolean showCellChief = false, showCellChampion = false, showCellStandard = false, showCellMusician = false, showCellUpgraded = false, showCellDisorganized = false, showCellState = false, showCellOrder = false;
		if (e instanceof Unit) {
			Unit u = (Unit) e;
			showCellChief = u.hasChief();
			showCellChampion = u.hasChampion();
			showCellStandard = u.hasStandardBearer();
			showCellMusician = (u.getMusicians() > 0);
			showCellUpgraded = u.isUpgraded();
			if (battleMode) {
				showCellDisorganized = u.isDisorganized();
				char order = u.getOrder();
				if (order == 'a') {
					cellOrder.setIcon(chargeIcon);
					cellOrder.setToolTipText(Resources.getString("charge.tt"));
				} else if (order == 'o') {
					cellOrder.setIcon(manoeuvreIcon);
					cellOrder.setToolTipText(Resources
							.getString("manoeuvre.tt"));
				} else if (order == 'h') {
					cellOrder.setIcon(holdIcon);
					cellOrder.setToolTipText(Resources.getString("defend.tt"));
				}
				showCellOrder = true;
			}
		}
		if (battleMode) {
			showCellState = e.getState() != 'n';
			if (e.getState() == 'a') {
				cellState.setIcon(annihilatedIcon);
				cellState.setToolTipText(Resources.getString("annihilated.tt"));
			} else if (e.getState() == 'r') {
				cellState.setIcon(routedIcon);
				cellState.setToolTipText(Resources.getString("routed.tt"));
			} else if (e.getState() == 's') {
				cellState.setIcon(shakenIcon);
				cellState.setToolTipText(Resources.getString("shaken.tt"));
			} else if (e.getState() == 'b') {
				cellState.setIcon(bloodLustIcon);
				cellState.setToolTipText(Resources.getString("bloodlust.tt"));
			}

		}
		cellChief.setVisible(showCellChief);
		cellChampion.setVisible(showCellChampion);
		cellStandard.setVisible(showCellStandard);
		cellMusician.setVisible(showCellMusician);
		cellUpgraded.setVisible(showCellUpgraded);
		cellDisorganized.setVisible(showCellDisorganized);
		cellState.setVisible(showCellState);
		cellOrder.setVisible(showCellOrder);
	}

	/**
	 * constructor
	 * 
	 * @param battleMode
	 *            renderer used in army editor or in battle information table ?
	 */
	public EntityStateRenderer(boolean battleMode) {
		super();
		cellChief.setToolTipText(Resources.getString("chief.tt"));
		cellChampion.setToolTipText(Resources.getString("champion.tt"));
		cellStandard.setToolTipText(Resources.getString("standard.tt"));
		cellMusician.setToolTipText(Resources.getString("musician.tt"));
		cellUpgraded.setToolTipText(Resources.getString("upgraded.tt"));
		cellDisorganized.setToolTipText(Resources.getString("disorganized.tt"));
		setLayout(new GridBagLayout());
		this.battleMode = battleMode;
		if (battleMode) {
			add(cellState, constraint);
			add(cellDisorganized, constraint);
			add(cellOrder, constraint);
		}
		add(cellChief, constraint);
		add(cellChampion, constraint);
		add(cellStandard, constraint);
		add(cellMusician, constraint);
		add(cellUpgraded, constraint);
		cellChief.setVisible(false);
		cellChampion.setVisible(false);
		cellStandard.setVisible(false);
		cellMusician.setVisible(false);
		cellUpgraded.setVisible(false);
		cellDisorganized.setVisible(false);
		cellState.setVisible(false);
		cellOrder.setVisible(false);
	}

}