/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.gui.widgets;

import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import fb.util.FBColors;

/**
 * a widget to choose a numerical value
 * 
 * @author farfpeuf
 * 
 */
public class SlideValueChooser extends JPanel {
	/**
	 * the value field
	 */
	private JTextField valueField;

	/**
	 * the slider to change the value
	 */
	private JSlider slider;

	/**
	 * the widget label
	 */
	private JLabel textLabel;

	/**
	 * constructor
	 * 
	 * @param text
	 *            the widget label
	 */
	public SlideValueChooser(String text) {
		this(text, false);
	}

	/**
	 * constructor
	 * 
	 * @param text
	 *            the widget label
	 * @param horizontal
	 *            is widget displayed horizontaly ?
	 */
	public SlideValueChooser(String text, boolean horizontal) {

		valueField = new JTextField();
		valueField.setHorizontalAlignment(JTextField.CENTER);
		valueField.setEditable(false);
		valueField.setForeground(FBColors.standardText);
		valueField.setBackground(FBColors.background);
		valueField.setBorder(null);
		slider = new JSlider(JSlider.HORIZONTAL, -100, 100, 5);

		slider.addChangeListener(new ChangeListener() {
			public void stateChanged(ChangeEvent e) {
				int value = ((JSlider) e.getSource()).getValue();
				valueField.setText(Integer.toString(value));
			}
		});

		textLabel = new JLabel(text);

		if (horizontal) {
			setLayout(new FlowLayout());
			slider.setMinimumSize(slider.getPreferredSize());
			add(textLabel);
			add(valueField);
			add(slider);
		} else {
			setLayout(new GridBagLayout());
			GridBagConstraints gbc = new GridBagConstraints();

			gbc.gridx = 0;
			gbc.gridy = 0;
			gbc.weightx = 1;
			gbc.anchor = GridBagConstraints.EAST;
			add(textLabel, gbc);
			gbc.gridx++;
			gbc.anchor = GridBagConstraints.WEST;

			valueField.setColumns(2);
			valueField.setMinimumSize(valueField.getPreferredSize());
			add(valueField, gbc);
			gbc.gridx--;
			gbc.gridy++;
			gbc.gridwidth = 2;
			gbc.fill = GridBagConstraints.HORIZONTAL;
			add(slider, gbc);
		}

		setSize(100, 40);
		validate();

	}

	/**
	 * set current minimun and maximum value
	 * 
	 * @param mn
	 *            the nminimum value
	 * @param mx
	 *            the maximum valur
	 * @param init
	 *            the initial value
	 */
	public void setValues(int mn, int mx, int init) {

		slider.setMinimum(mn);
		slider.setMaximum(mx);
		slider.setValue(init);
	}

	/**
	 * Adds a ChangeListener to the slider.
	 * 
	 * @param l
	 *            the ChangeListener to add
	 */
	public void addChangeListener(ChangeListener changeListener) {
		slider.addChangeListener(changeListener);
	}

	/**
	 * change label text
	 * 
	 * @param text
	 *            the new text
	 */
	public void setText(String text) {
		textLabel.setText(text);
	}
}
