/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.phases;

import java.util.Vector;

import fb.army.entities.Entity;
import fb.army.entities.Unit;
import fb.army.hierarchy.Army;
import fb.gui.frames.BattleFrame;
import fb.main.FB;
import fb.main.Battle;
import fb.network.Client;
import fb.sound.SoundPlayer;

/**
 * close combat phase
 * 
 * @author farfpeuf
 * 
 */
public class CombatPhase extends Phase {
	/**
	 * phase name
	 */
	private static final String name = "combatphase";

	/**
	 * phase icon number
	 */
	private static final int iconNumber = 3;

	/**
	 * entities fighting during this phase
	 */
	private Vector fighters;

	/**
	 * is phase finished ?
	 */
	private boolean finished;

	/**
	 * initialize this phase
	 * 
	 */
	public void init() {
		fighters = new Vector();
		setWithPhasePanel(true);
		finished = false;
		super.init();
	}

	/**
	 * @return phase name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return phase icon number
	 */
	public int getIconNumber() {
		return iconNumber;
	}

	/**
	 * @return phase toolip
	 */
	public String getTooltip() {
		return null;// no tooltip
	}

	/**
	 * @return true if phase is finished
	 */
	public boolean finished() {
		return finished;
	}

	/**
	 * proceed to phase next step
	 */
	public void nextStep() {
		FB.setWaitCursor();
		Battle.getInstance().setArmy(null);
		Entity entity;
		BattleFrame.getInstance().setPhase(this);
		for (int i = 0; !stop
				&& i < Battle.getInstance().getActiveEntities().size(); i++) {
			entity = (Entity) (Battle.getInstance().getActiveEntities().get(i));
			entity.calculateContacts();
			if (entity.getOpposingContacts().size() > 0) {
				if (entity.isHandled()) {
					entity.fight();
				}
				if (!fighters.contains(entity)) {
					fighters.add(entity);
				}
			}
		}
		if (FB.isNet()) {
			Client.getInstance().synchronize();
		}
		finished = true;
	}

	/**
	 * end this phase
	 * 
	 */
	public void end() {
		if (fighters.size() != 0) {
			Entity entity;
			SoundPlayer.getInstance().play("fight");
			if (FB.getInstance().isGraphicMode()) {
				for (int i = 0; !stop && i < fighters.size(); i++) {
					((Entity) fighters.get(i)).animFight();
				}
				if (!FB.getInstance().isBatch()) {
					sleep(2000);
				}
				for (int i = 0; !stop && i < fighters.size(); i++) {
					((Entity) fighters.get(i)).animKill();
				}
				if (!FB.getInstance().isBatch()) {
					sleep(2000);
				}
				for (int i = 0; i < fighters.size(); i++) {
					entity = (Entity) (fighters.get(i));
					entity.stand();
				}
			}

			for (int i = 0; i < fighters.size(); i++) {
				entity = (Entity) (fighters.get(i));

				entity.commitKill();
				if (entity.getType() != 'u' && entity.getState() != 'a') {
					if (entity.isUnit() && entity.isHandled()) {
						Unit u = (Unit) entity;
						u.combatTest();
					}
				}
			}
			if (FB.isNet()) {
				Client.getInstance().synchronize();
			}
			Army[] armies = Battle.getInstance().getArmies();
			for (int i = 0; i < armies.length; i++) {
				if (armies[i].mustTestMorale() && armies[i].isHandled()) {
					armies[i].moraleTest();
				}
			}
			if (FB.isNet()) {
				Client.getInstance().synchronize();
			}
			for (int i = 0; i < Battle.getInstance().getActiveEntities().size(); i++) {
				entity = (Entity) Battle.getInstance().getActiveEntities().get(
						i);
				// attendre la fin des mouvements de deroute et de
				// reorganisation
				while (!stop && entity.isWalking()) {
					sleep(125);
				}
				if (entity.getState() == 'r') {
					entity.commitRout();
				}
			}
			if (FB.isNet()) {
				Client.getInstance().synchronize();
			}
			for (int i = 0; i < fighters.size(); i++) {
				entity = (Entity) (fighters.get(i));
				if (entity.isUnit()) {
					Unit u = (Unit) entity;
					if (u.isSurrounding()) {
						if (u.isActive()) {
							if (u.getOpposingContacts().size() == 0) {
								u.setSurrounding(false);
								u.calculateWarriorsPositions();
							} else {
								u.surround();
							}
						}
					}
				}
			}
			for (int i = 0; i < Battle.getInstance().getActiveEntities().size(); i++) {
				entity = (Entity) Battle.getInstance().getActiveEntities().get(
						i);
				// attendre la fin des mouvements de reorganisation
				while (!stop && entity.isWalking()) {
					sleep(125);
				}
			}
		}
	}
}