/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.phases;

import fb.army.hierarchy.Army;
import fb.main.FB;
import fb.main.Battle;
import fb.network.Client;
import fb.network.MessagesConstants;
import fb.util.DX;
import fb.util.NetworkDX;

/**
 * a game phase synchronized on the network
 * 
 * @author farfpeuf
 * 
 */
public abstract class SynchronizedPhase extends Phase {

	/**
	 * current army index
	 */
	protected int current;

	/**
	 * implied armies
	 */
	protected Army[] armies;

	/**
	 * @return current playing army
	 */
	public Army getCurrentArmy() {
		if (armies == null || current >= armies.length) {
			return null;
		}
		return armies[current];
	}

	/**
	 * constructor get implied armies
	 * 
	 */
	public SynchronizedPhase() {
		armies = Battle.getInstance().getArmies();
	}

	/**
	 * initialize this phase
	 * 
	 */
	public void init() {
		initCurrent();
		super.init();
	}

	/**
	 * @return true if phase is finished
	 */
	public boolean finished() {
		return Battle.getLoop().getNumberOfPhaseTokens() == armies.length;
	}

	/**
	 * change current playing army
	 * 
	 */
	public void changeArmy() {
		int i = current + 1;
		if (i == armies.length) {
			i = 0;
		}
		while (Battle.getLoop().getPhaseTokens()[i] && !finished()) {
			i++;
			if (i == armies.length) {
				i = 0;
			}
		}
		current = i;
		if (!finished()) {
			Battle.getInstance().setArmy(armies[current]);
		} else {
			Battle.getInstance().setArmy(null);
		}

	}

	/**
	 * determine first playing army
	 * 
	 */
	public void initCurrent() {
		if (FB.isNet()) {
			current = NetworkDX.roll(armies.length) - 1;
		} else {
			current = DX.roll(armies.length) - 1;
		}
		int i = current;
		boolean stop = false;
		while (!stop) {
			i++;
			if (i == armies.length) {
				i = 0;
			}
			if (armies[i].isHandled()) {
				stop = true;
			}
		}
		current = i;
		Battle.getInstance().setArmy(armies[current]);

	}

	/**
	 * set token to specify that an army has finished to play phase
	 * 
	 * @param armyId
	 *            the army index
	 */
	public void setToken(int armyId) {
		if (FB.isNet() && armies[current].isHandled()) {
			Client.getInstance().sendAndWait(MessagesConstants.DONE + current);
		} else {
			Battle.getLoop().doSetToken(current);
		}
	}

	/**
	 * phase waiting for network action ?
	 */

	public boolean isWaitingForNetAction() {
		return !armies[current].isHandled()
				&& !Battle.getLoop().getPhaseTokens()[current];
	}

	/**
	 * wait for network action
	 */
	protected void waitNetAction() {
		FB.setWaitCursor();
		if (!stop) {
			if (Battle.getLoop().getPhaseTokens()[current]) {
				if (!finished()) {
					changeArmy();
				}
			}
		}
	}

}