/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.phases;

import fb.army.entities.Entity;
import fb.army.entities.Unit;
import fb.artificialintelligence.TargetFinder;
import fb.main.FB;
import fb.network.Client;
import fb.sound.SoundPlayer;

/**
 * intimidation phase
 * 
 * @author farfpeuf
 * 
 */
public class ThreatPhase extends BinaryPhase {
	/**
	 * phase name
	 */
	private static final String name = "threatphase";

	/**
	 * phase icon number
	 */
	private static final int iconNumber = 0;

	/**
	 * constructor
	 * 
	 */
	public ThreatPhase() {
		super();
		setWithPhasePanel(true);
	}

	/**
	 * @return phase name
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return phase icon number
	 */
	public int getIconNumber() {
		return iconNumber;
	}

	/**
	 * is this entity a potential target for specified actor
	 * 
	 * @param a
	 *            phase actor
	 * @param e
	 *            phase target
	 * @return true if this entity is a potential target for specified actor
	 */
	public boolean isTarget(Entity actor, Entity e) {
		if (e.getType() != 'u') {
			if (e.isUnit()) {
				if (actor.lineOfSightTo(e) < 12 * 8) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * is this entity a potential target for current actor
	 * 
	 * @param e
	 *            phase target
	 * @return true if this entity is a potential target for current actor
	 */
	public boolean isTarget(Entity e) {
		if (e.getType() != 'u') {
			if (e.isUnit()) {
				if (getActor().lineOfSightTo(e) < 12 * 8) {
					return true;
				}
			}
		}
		return false;
	}

	/**
	 * is this entity a potential actor for this phase
	 * 
	 * @param a
	 *            phase actor
	 * @return true if this entity is a potential actor
	 */
	public boolean isActor(Entity e) {
		if (e.isUnit()) {
			if (e.getType() != 'c') {
				if (e.getState() != 's') {
					if (e.getState() != 'r') {
						if (e.getOpposingContacts().size() == 0) {
							return true;
						}
					}
				}
			}
		}
		return false;

	}

	/**
	 * calculate phase results
	 * 
	 */
	public void calculateResults() {

		if (actors.size() != 0) {

			SoundPlayer.getInstance().play("threat");
			if (FB.getInstance().isGraphicMode()) {
				for (int i = 0; i < actors.size(); i++) {
					Unit act = (Unit) actors.get(i);
					act.animThreat((Unit) targets.get(i));
				}
				if (!FB.getInstance().isBatch()) {
					sleep(3000);
				}

			}
			for (int i = 0; i < actors.size(); i++) {
				Unit act = (Unit) actors.get(i);
				if (act.isHandled()) {
					Unit tar = (Unit) targets.get(i);
					act.threat(tar);
				}
				act.stand();
			}
			if (FB.isNet()) {
				Client.getInstance().synchronize();
			}

		}

	}

	/**
	 * @return AI thread that handles choice of the best target among a set of
	 *         availables targets
	 */
	public TargetFinder getTargetFinder() {
		return new TargetFinder(getPotentialTargets(), 3, getActor()) {

			protected int getWeight(Object element) {
				Entity potentialTarget = (Entity) element;
				int poid = super.getWeight(potentialTarget);

				Unit u = (Unit) entity;

				poid += (10 - u.threatScore((Unit) potentialTarget)/15);

				if (potentialTarget == getClosestTarget()) {
					poid += 2;
				}

				return poid;
			}

		};

	}

}
