/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.sound;

import java.io.File;

import javax.sound.midi.MetaEventListener;
import javax.sound.midi.MetaMessage;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Sequence;
import javax.sound.midi.Sequencer;

import fb.main.FB;
import fb.util.DX;

/**
 * handle midi music files playing
 * 
 * @author farfpeuf
 * 
 */
public class MusicHandler {
	/**
	 * the midi sequencer
	 */
	private static Sequencer sequencer;

	/**
	 * the available tracks
	 */
	private static File[] tracks;

	/**
	 * the end of track listener
	 */
	private static MetaEventListener stopListener = new MetaEventListener() {
		public void meta(MetaMessage meta) {
			if (meta.getType() == 47) {
				playRandomTrack();
			}

		}
	};

	/**
	 * set available tracks
	 * 
	 * @param names
	 *            the available tracks names
	 */
	public static void setTracks(String[] names) {
		tracks = new File[names.length];
		for (int i = 0; i < names.length; i++) {
			tracks[i] = new File("music/" + names[i]);
		}
	}

	/**
	 * get sequancer instance
	 * 
	 * @return the sequencer
	 * @throws MidiUnavailableException
	 */
	private static synchronized Sequencer getSequencerInstance()
			throws MidiUnavailableException {
		if (null == sequencer) {
			sequencer = MidiSystem.getSequencer();
			sequencer.open();
		}
		return sequencer;
	}

	/**
	 * play an available track chosen at random
	 * 
	 */
	public static void playRandomTrack() {
		play(DX.roll(tracks.length) - 1);
	}

	/**
	 * play a track
	 * 
	 * @param track
	 *            the track id
	 */
	public static void play(int track) {
		try {
			getSequencerInstance().removeMetaEventListener(stopListener);
			loadTrack(track);
			getSequencerInstance().setTickPosition(0);
			getSequencerInstance().start();
			getSequencerInstance().addMetaEventListener(stopListener);
		} catch (Exception e) {
			FB.exception(e);
		}
	}

	/**
	 * load a track
	 * 
	 * @param track
	 *            the track id
	 * @throws Exception
	 */
	private static void loadTrack(int track) throws Exception {
		Sequence sequence = MidiSystem.getSequence(tracks[track]);
		getSequencerInstance().setSequence(sequence);

	}

	/**
	 * stop music playing
	 * 
	 */
	public static void stop() {
		try {
			getSequencerInstance().stop();
		} catch (MidiUnavailableException e) {
			FB.exception(e);
		}
	}

}