/*
 * Copyright 2002 Farfpeuf/ AdetSoft. All rights reserved.
 * Use is subject to license terms.
 */
package fb.sound;

import javax.sound.sampled.Clip;

/**
 * @author farfpeuf 
 * A class to play sounds
 */

public class SoundPlayer {

	private int nblaunched = 0;

	/**
	 * the available sounds list
	 */

	private SoundList soundList;

	/**
	 * the unique instance of this class
	 */

	private static SoundPlayer instance;

	/**
	 * Constructs the unique instance of this class
	 */

	private SoundPlayer() {
		soundList = new SoundList();
	}

	/**
	 * returns the unique instance of this class
	 */

	public static synchronized SoundPlayer getInstance() {
		if (instance == null) {
			instance = new SoundPlayer();
		}
		return instance;
	}

	/**
	 * plays a sound of the specified type if one is available
	 * 
	 * @param soundType
	 *            the type of sound to play.
	 */

	public void play(String soundType) {

		Clip clip = soundList.getClip(soundType);
		if (clip != null) {
			clip.start();
		}
	}

	/**
	 * creates and starts a LineLoader for this type of sound
	 * 
	 * @param soundType
	 *            the type of sound to load.
	 */

	public void load(String soundType) {
		new LineLoader(soundList, soundType).start();
		nblaunched++;
	}

	public void loadall() {

		load("intro"); // 1 chanel
		load("attack"); // 1 chanel
		load("hold"); // 1 chanel
		load("oppose"); // 1 chanel
		load("threat"); // 1 chanel

		load("shaken"); // 3 chanels
		load("bloodlust");
		load("rout");

		load("fight"); // 1 chanel

		load("magie"); // 4 chanels
		load("magie");
		load("magiecast");
		load("magiecast");

		load("right"); // 10 chanels
		load("left");
		load("walk");
		load("indvdwalk");
		load("position");

		load("arrow"); // 3 chanels
		load("arrow");
		load("arrow");

		load("death"); // 3 chanels
		load("death");
		load("death");

	}

	/**
	 * calculate sound loading progress
	 * 
	 * @return sound loading percentage
	 */
	public int progress() {
		if (soundList != null && nblaunched != 0) {
			return (int) (100 * soundList.nbready / nblaunched);
		} else {
			return 0;
		}
	}

}
